-- 
-- Oracle Schema:  CREATE PROCEDURES
--

CREATE OR REPLACE PROCEDURE AddNewStudent (
  p_FirstName  students.first_name%TYPE,
  p_LastName   students.last_name%TYPE,
  p_Major      students.major%TYPE DEFAULT 'Economics') AS
BEGIN
  -- Insert a new row in the students table. Use
  -- student_sequence to generate the new student ID, and
  -- 0 for current_credits.
  INSERT INTO students VALUES (student_sequence.nextval,
    p_FirstName, p_LastName, p_Major, 0);

  COMMIT;
END AddNewStudent;
/


CREATE OR REPLACE PROCEDURE CalculateGPA (
  /* Returns the grade point average for the student identified
     by p_StudentID in p_GPA. */
  p_StudentID IN students.ID%TYPE,
  p_GPA OUT NUMBER) AS

  CURSOR c_ClassDetails IS
    SELECT classes.num_credits, rs.grade
      FROM classes, registered_students rs
      WHERE classes.department = rs.department
      AND classes.course = rs.course
      AND rs.student_id = p_StudentID;

  v_NumericGrade NUMBER;
  v_TotalCredits NUMBER := 0;
  v_TotalGrade NUMBER := 0;

BEGIN
  FOR v_ClassRecord in c_ClassDetails LOOP
    -- Determine the numeric value for the grade.
    SELECT DECODE(v_ClassRecord.grade, 'A', 4,
                                       'B', 3,
                                       'C', 2,
                                       'D', 1,
                                       'E', 0)
      INTO v_NumericGrade
      FROM dual;

    v_TotalCredits := v_TotalCredits + v_ClassRecord.num_credits;
    v_TotalGrade := v_TotalGrade +
                    (v_ClassRecord.num_credits * v_NumericGrade);
  END LOOP;

  p_GPA := v_TotalGrade / v_TotalCredits;
END CalculateGPA;
/


CREATE OR REPLACE PROCEDURE InsertIntoTemp AS
  /* Declare variables to be used in this block. */
  v_Num1      NUMBER := 5;
  v_Num2      NUMBER := 6;
  v_String1   VARCHAR2(50) := 'Hello World!';
  v_String2   VARCHAR2(50) := '-- This message brought to you by PL/SQL!';
  v_OutputStr VARCHAR2(50);
BEGIN
  /* First, insert two rows into temp_table, using the values of the
     variables. */
  INSERT INTO temp_table (num_col, char_col)
    VALUES (v_Num1, v_String1);
  INSERT INTO temp_table (num_col, char_col)
    VALUES (v_Num2, v_String2);

  /* Now query temp_table for the two rows we just inserted, and output
     them to the screen using the DBMS_OUTPUT package. */
  SELECT char_col
    INTO v_OutputStr
	FROM temp_table
	WHERE num_col = v_Num1;
  DBMS_OUTPUT.PUT_LINE(v_OutputStr);

  SELECT char_col
    INTO v_OutputStr
	FROM temp_table
	WHERE num_col = v_Num2;
  DBMS_OUTPUT.PUT_LINE(v_OutputStr);
END InsertIntoTemp;
/


CREATE OR REPLACE PROCEDURE RecreateTempTable (
  /* Drops temp_table and re-creates it. The table description
     is passed in with p_Description, and should be the contents
     of the CREATE TABLE statement, after the table name. For
     example, the following is a legal call:
     RecreateTempTable('(num_col NUMBER, char_col VARCHAR2(2000))');
 */
  p_Description IN VARCHAR2) IS
  v_CreateString  VARCHAR2(100);
  v_DropString    VARCHAR2(100);
BEGIN
  /* Drop the table first. */
  v_DropString := 'DROP TABLE temp_table';
  /* Execute the 'DROP TABLE' command. Trap the ORA-942 error in
     case the table doesn't yet exist. */
  BEGIN
    EXECUTE IMMEDIATE v_DropString;
  EXCEPTION
    WHEN OTHERS THEN
      IF SQLCODE != -942 THEN
        RAISE;
      END IF;
  END;
  /* Now re-create it. We need to create the CREATE TABLE
     string first, then we can execute it. */
  v_CreateString := 'CREATE TABLE temp_table ' || p_Description;
  EXECUTE IMMEDIATE v_CreateString;
END RecreateTempTable;
/


CREATE OR REPLACE PROCEDURE CreditLoop AS
  /* Inserts the student ID numbers and their current credit
     values into temp_table. */
  v_StudentID students.ID%TYPE;
  v_Credits   students.current_credits%TYPE;
  CURSOR c_Students IS
    SELECT ID
      FROM students;
BEGIN
  OPEN c_Students;
  LOOP
    FETCH c_Students INTO v_StudentID;
    v_Credits := CountCredits(v_StudentID);
    INSERT INTO temp_table (num_col, char_col)
      VALUES (v_StudentID, 'Credits = ' || TO_CHAR(v_Credits));
    EXIT WHEN c_Students%NOTFOUND;
  END LOOP;
  CLOSE c_Students;
END CreditLoop;
/


CREATE OR REPLACE PROCEDURE CopyTables AS
  v_Key   source.key%TYPE;
  v_Value source.value%TYPE;

  CURSOR c_AllData IS
    SELECT *
      FROM source;
BEGIN
  OPEN c_AllData;

  LOOP
    FETCH c_AllData INTO v_Key, v_Value;
    EXIT WHEN c_AllData%NOTFOUND;

    INSERT INTO destination (key, value)
      VALUES (v_Key, TO_NUMBER(v_Value));
  END LOOP;

  CLOSE c_AllData;
END CopyTables;
/


CREATE OR REPLACE PROCEDURE TempInsert
  (p_NextDate IN OUT DATE) AS
  v_SeqNum   NUMBER;
  v_StartNum NUMBER;
  v_SQLCode NUMBER;
  v_Errmsg  VARCHAR2(60);
BEGIN
  SELECT temp_seq.nextval
    INTO v_SeqNum
    FROM dual;
  -- See if this is the first time we're called
  BEGIN
    SELECT num_col
      INTO v_StartNum
      FROM temp_table
      WHERE char_col = 'TempInsert Start';

    -- We've been called before, so insert a new value
    INSERT INTO temp_table (num_col, char_col)
      VALUES (v_SeqNum, TO_CHAR(SYSDATE, 'DD-MON-YY HH24:MI:SS'));
  EXCEPTION
    WHEN NO_DATA_FOUND THEN
      -- First time we're called, so insert
      INSERT INTO temp_table (num_col, char_col)
        VALUES (v_SeqNum, 'TempInsert Start');
  END;

  -- If we've been called more than 15 times, exit.
  IF v_SeqNum - V_StartNum > 15 THEN
    p_NextDate := NULL;
  END IF;

  COMMIT;
END TempInsert;
/


CREATE OR REPLACE PROCEDURE Register (
  /* Registers the student identified by the p_StudentID parameter in the class
     identified by the p_Department and p_Course parameters. Before calling
     ClassPackage.AddStudent, which actually adds the student to the class, this
     procedure verifies that there is room in the class, and that the class
     exists. */
  p_StudentID IN students.id%TYPE,
  p_Department IN classes.department%TYPE,
  p_Course IN classes.course%TYPE) AS

  v_CurrentStudents NUMBER;  -- Current number of students in the class
  v_MaxStudents NUMBER;      -- Maximum number of students in the class

BEGIN
  /* Determine the current number of students registered, and the maximum
     number of students allowed to register. */
  SELECT current_students, max_students
    INTO v_CurrentStudents, v_MaxStudents
    FROM classes
    WHERE course = p_Course
    AND department = p_Department;

  /* Make sure there is enough room for this additional student. */
  IF v_CurrentStudents + 1 > v_MaxStudents THEN
    RAISE_APPLICATION_ERROR(-20000, 'Can''t add more students to ' ||
      p_Department || ' ' || p_Course);
  END IF;

  /* Add the student to the class. */
  ClassPackage.AddStudent(p_StudentID, p_Department, p_Course);

EXCEPTION
  WHEN NO_DATA_FOUND THEN
    /* Class information passed to this procedure doesn't exist. Raise an error
       to let the calling program know of this. */
    RAISE_APPLICATION_ERROR(-20001, p_Department || ' ' || p_Course ||
      ' doesn''t exist!');
END Register;
/
